<?php

class ESLImageConvertor {
	private $previous_x = 0;
	private $previous_y = 0;
	private $numOfNots = 0;

	public function __construct() {

	}

	public function convertImage($filepath, $rgbThreshold) {
		$resource = imagecreatefromstring(file_get_contents($filepath));
		$width = imagesx($resource);
		$height = imagesy($resource);
		$factor = max(1, $width / 500, $height / 500);
		$handdrawn = '';
		
		
		do{		
			if($factor > 1){
				$newwidth = $width / $factor;
				$newheight = $height / $factor;

				$thumb = imagecreatetruecolor($newwidth, $newheight);
				  
				imagecopyresized($thumb, $resource, 0, 0, 0, 0, $newwidth, $newheight, $width, $height);
				
				$handdrawn = $this->doConvert($thumb, $rgbThreshold);
			} else{
				$handdrawn = $this->doConvert($resource, $rgbThreshold);
			}
			
			
			$factor += 0.3;
		}while(strlen($handdrawn) > 65535 / 2);
		

		return $handdrawn;
	}

	public function doConvert($resource, $rgbThreshold) {

		$width = imagesx($resource);
		$height = imagesy($resource);

		$pixel_x = 0;
		$startX = 0;
		
		$vectorResult = array();

		for ($y = 0; $y < $height; $y++) {
			$previous = false;

			for ($x = 0; $x < $width; $x++) {
				$rgb = imagecolorat($resource, $x, $y);
				$r = ($rgb >> 16) & 0xFF;
				$g = ($rgb >> 8) & 0xFF;
				$b = $rgb & 0xFF;
				$drawPixel = (($r + $g + $b) / 3 <= $rgbThreshold) ? true : false;


				if ($drawPixel && $previous) {
					$pixel_x = $x;
				} else if ($drawPixel) {
					$pixel_x = $x;
					
					$startX = $x;
					$previous = true;
				} else {
					if ($previous) {
						  if(($pixel_x - $startX) >= 2) {
							  $vectorResult = array_merge($vectorResult, $this->parseStroke($startX, $y, false, $vectorResult));
							  $vectorResult = array_merge($vectorResult, $this->parseStroke(floor(($startX +$pixel_x)/2), $y, true, $vectorResult));
							  $vectorResult = array_merge($vectorResult, $this->parseStroke($pixel_x, $y, true, $vectorResult));
                          }else if(($pixel_x - $startX) == 1){							  
							  $vectorResult = array_merge($vectorResult, $this->parseStroke($startX, $y, false, $vectorResult));							  
							  $vectorResult = array_merge($vectorResult, $this->parseStroke($pixel_x, $y, false, $vectorResult));
                          }else if($pixel_x == $startX){							 
							  $vectorResult = array_merge($vectorResult, $this->parseStroke($startX, $y, false, $vectorResult));
                          }
                          $previous = false;			
					}
				}

			}
		}

		$resultArray = array();

		array_push($resultArray, ($width & 0xFF));
		array_push($resultArray, (($width >> 8) & 0xFF));
		array_push($resultArray, ($height & 0xFF));
		array_push($resultArray, (($height >> 8) & 0xFF));
		//{0x03, 0x00}
		array_push($resultArray, 0x03);
		array_push($resultArray, 0x00);
		array_push($resultArray, 0x03);
		array_push($resultArray, 0x00);

		array_push($resultArray, ((count($vectorResult) + 12) & 0xFF));
		array_push($resultArray, (((count($vectorResult) + 12) >> 8) & 0xFF));
		array_push($resultArray, (((count($vectorResult) + 12) >> 16) & 0xFF));
		array_push($resultArray, (((count($vectorResult) + 12) >> 24) & 0xFF));

		array_push($resultArray, 0x03);
		array_push($resultArray, 0x00);
		array_push($resultArray, 0x00);
		array_push($resultArray, 0x00);

		array_push($resultArray, ($width & 0xFF));
		array_push($resultArray, (($width >> 8) & 0xFF));
		array_push($resultArray, ($height & 0xFF));
		array_push($resultArray, (($height >> 8) & 0xFF));

		array_push($resultArray, ($this->numOfNots & 0xFF));
		array_push($resultArray, (($this->numOfNots >> 8) & 0xFF));
		array_push($resultArray, (($this->numOfNots >> 16) & 0xFF));
		array_push($resultArray, (($this->numOfNots >> 24) & 0xFF));

		$resultArray = array_merge($resultArray, $vectorResult);

		$afterMD5 = $this->javaMd5($resultArray);

		$result = array();
		array_push($result, 0x01);
		array_push($result, 0x00);
		array_push($result, 0x00);
		array_push($result, 0x00);
		$result = array_merge($result, $afterMD5);
		$result = array_merge($result, $resultArray);

		$result_new = array_map(function ($value) {return chr($value);}, $result);

		$joinString = join("", $result_new);

		return base64_encode($joinString);
	}

	private function parseStroke($x, $y, $input_pen) {

		$vectorResult = array();
		$delx = abs($this->previous_x - $x);
		$dely = abs($this->previous_y - $y);
		$maximum = max($delx, $dely);
		$packet_type = 0;
		$pen = 0;
		$signx = 0;
		$signy = 0;

		if ($input_pen) {
			// fits in 2 bits
			if ($maximum <= 0x0003) {
				$packet_type = 1;
			}

			// fits in 6 bits
			else if ($maximum <= 0x003f) {
				$packet_type = 2;
			}

			// fits in 10 bits
			else if ($maximum <= 0x03ff) {
				$packet_type = 3;
			}

			// fits in 12 bits
			else if ($maximum <= 0x0fff) {
				$packet_type = 4;
			} else {
				$packet_type = 5;
			}

			$pen = 0x01;
		} else {
			// fits in 4 bits
			if ($maximum <= 0x000f) {
				$packet_type = 6;
			}

			// fits in 8 bits
			else if ($maximum <= 0x00ff) {
				$packet_type = 7;
			}

			// fits in 12 bits
			else if ($maximum <= 0x0fff) {
				$packet_type = 4;
			} else {
				$packet_type = 5;
			}

			$pen = 0x00;
		}
		// set sign bit
		if ($this->previous_x <= $x) {
			$signx = 0;
		} else {
			$signx = 0x20;
		}

		if ($this->previous_y <= $y) {
			$signy = 0;
		} else {
			$signy = 0x10;
		}

		$this->numOfNots++;

		switch ($packet_type) {
		case 1:
			array_push($vectorResult, (0x80 |
				$signx |
				$signy |
				(($delx & 0x03) << 2) |
				($dely & 0x03)));
			break;
		case 2:
			array_push($vectorResult, (0x40 |
				$signx |
				$signy |
				($delx >> 2 & 0x0c) |
				($dely >> 4 & 0x03)));
			array_push($vectorResult, ((($delx << 4) & 0xf0) |
				($dely & 0x0f)));

			break;
		case 3:
			array_push($vectorResult, (0xc0 |
				$signx |
				$signy |
				($delx >> 6 & 0x0c) |
				($dely >> 8 & 0x03)));
			array_push($vectorResult, ($delx & 0xff));
			array_push($vectorResult, ($dely & 0xff));
			break;
		case 6:
			array_push($vectorResult, ($signx | $signy));
			array_push($vectorResult, ((($delx << 4) & 0xf0) |
				($dely & 0x0f)));

			break;
		case 7:
			array_push($vectorResult, ($signx | $signy | 0x02));
			array_push($vectorResult, ($delx & 0xff));
			array_push($vectorResult, ($dely & 0xff));
			break;
		case 4:
			array_push($vectorResult, ($signx | $signy | 0x04 | $pen));
			array_push($vectorResult, (($delx >> 4 & 0xf0) |
				($dely >> 8 & 0x0f)));
			array_push($vectorResult, ($delx & 0xff));
			array_push($vectorResult, ($dely & 0xff));

			break;
		case 5:
			array_push($vectorResult, ($signx | $signy | 0x06 | $pen));
			array_push($vectorResult, ($delx >> 8 & 0xff));
			array_push($vectorResult, ($delx & 0xff));
			array_push($vectorResult, ($dely >> 8 & 0xff));
			array_push($vectorResult, ($dely & 0xff));

			break;
		default:
			break;

		}

		$this->previous_x = $x;
		$this->previous_y = $y;

		return $vectorResult;

	}

	private function javaMd5($data) {
		assert(is_array($data));

		$dataString = $this->byteArrayToString($data);

		$hashString = md5($dataString, true);

		$hash = $this->stringToByteArray($hashString);

		return $hash;
	}

	private function stringToByteArray($s) {
		$result = array_fill(0, strlen($s), 0);
		for ($i = 0; $i < strlen($s); $i++) {
			$result[$i] = ord($s[$i]);
		}
		return $result;
	}

	private function byteArrayToString($b) {
		$asciiString = '';
		for ($i = 0; $i < count($b); $i++) {
			assert($b[$i] >= 0 && $b[$i] <= 255);
			$asciiString .= chr($b[$i]);
		}

		$utf8String = utf8_encode($asciiString);

		return $utf8String;
	}

}

$filepath = "...\\handdrawn5.png";
$handdrawn = (new ESLImageConvertor())->convertImage($filepath, 140);
echo ($handdrawn);



?>





